;****************************************************************************
;* DPC+ TEST PROGRAM
;****************************************************************************
    processor 6502
    include "vcs.h"
    include "macro.h"
    include "DPCplus.h"        
     
  MAC JUMP_TABLE                    ;Placed at the start of every bank
    RORG $F000                      ;-
    DC.L $E1A0F00E                  ;User ARM code stub - MOV PC,LR        
    align 128                       ;Reserved space for DPC+ registers (arm stub overlaps it)
InitSystem:
Bank0:
  	cmp SelectBank0
  	jmp Bank0Code 
Bank1:
  	cmp SelectBank1
  	jmp Bank1Code 
Bank2:
  	cmp SelectBank2
  	jmp Bank2Code 
Bank3:
  	cmp SelectBank3
  	jmp Bank3Code 
Bank4:
  	cmp SelectBank4
  	jmp Bank4Code 
Bank5:
    cmp SelectBank5
    jmp Bank5Code
  ENDM
        
  MAC BANKS_AND_VECTORS             ;Placed at the end of every bank
    RORG $FFF6
SelectBank0 .byte $00
SelectBank1 .byte $00
SelectBank2 .byte $00
SelectBank3 .byte $00
SelectBank4 .byte $00               ;NMI - overlaps with SelectBank4/5
SelectBank5 .byte $00               ;-
    .word InitSystem                ;RESET
    .word InitSystem                ;IRQ
  ENDM

;--------------------------------------------------------------------------------------------------
; [***START OF ROM***]
;--------------------------------------------------------------------------------------------------
  ;INCBIN "DPCplus.arm"              ;3K arm Stub code for DPC+, not working here like it should
  
  SEG.u VARS                         ;Segment for bank code
  ORG $80
;--------------------------------------------------------------------------------------------------
; [VARIABLES]
;--------------------------------------------------------------------------------------------------
;********************************
;     [POSITION VARS] 2-bytes
;********************************
Player0X  ds 1   ;Player position X 0-95
Player0Y  ds 1   ;Player position Y 0-191

;********************************
;     [KERNAL DATA] 3-bytes
;********************************
ScanCount ds 1  ;Scanline count
FrameNum  ds 1  ;Which frame we are on currently
P0pointer ds 2  ;Player Postion pointer
SPRpoint  ds 2  ;Sprite pointer

  SEG CODE                          ;Switch segment
;--------------------------------------------------------------------------------------------------
; [ROM BANKS]
;--------------------------------------------------------------------------------------------------  
;*********************
; [START OF BANK 0]
;*********************
  ORG  $0000                        ;Relocatable ORG address
  RORG $1000                        ;-  
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank0Code:
; --------------------------------------------------------------------------------------------------
; [INITIALIZATION]
; --------------------------------------------------------------------------------------------------
RESET:
    CLEAN_START ;macro.h
            ;Clear DPC ScreenRAM
            lda #<ScreenRAM1
            sta DF1LOW
            lda #>ScreenRAM1
            sta DF1HI        
            lda #<ScreenRAM2
            sta DF2LOW
            lda #>ScreenRAM2
            sta DF2HI
            lda #0
            ldy #6                ;6 bytes wide per ScreenRAM
            ldx #192              ;# of Scanlines tall
.loadRAM    sta DF1WRITE          ;Clear ScreenRAM1
            sta DF2WRITE          ;Clear ScreenRAM2
            dex
            bne .loadRAM
            ldx #192
            dey
            bne .loadRAM    
            
;********************************
;     [SETUP STARTUP STATE]
;********************************    
            lda #$00                ;Set data direction registers to input state
            sta SWACNT              ;-
            sta SWBCNT              ;-
            lda #$03                ;Set P0/1 to 3 copies
            sta NUSIZ0              ;-
            sta NUSIZ1              ;-
            lda #$16
            sta COLUP0
            sta COLUP1
            
            ;***START KERNAL LOOP***
; --------------------------------------------------------------------------------------------------
; [KERNAL]/[VSYNC] ;3 scanlines of VSYNC
; --------------------------------------------------------------------------------------------------
KERNAL:     lda #%1110              ;[]+2   ;Scanline [258], S.cyc [3],
VS_LOOP:    sta WSYNC               ;[]+3   ;PixelPos [-59], Color clock [3]        
            sta VSYNC               ;[]+3   ;Each '1' bits generates
            lsr                     ;[]+2   ;-a VSYNC ON line (bits 1..3)
            bne VS_LOOP             ;[]+2   ;-1st '0' bit resets Vsync
            lda #44                 ;[]+2   ;Set timer to 37 scanlines
            sta TIM64T              ;[]+4   ;worth of time ((37*76)/64)
; --------------------------------------------------------------------------------------------------
; [VBLANK]  ;2812 Cycles for use
;           ;76 per scanline
;           ;37 scanlines VBLANK
; --------------------------------------------------------------------------------------------------
;********************************
;     [37 SCANLINES OF LOGIC]
;********************************
            ;RESET DataFetchers pointers
            lda #<ScreenRAM1
            sta DF1LOW
            lda #>ScreenRAM1
            sta DF1HI        
            lda #<ScreenRAM2
            sta DF2LOW
            lda #>ScreenRAM2
            sta DF2HI      
;********************************
;     [READY TO START KERNAL]
;********************************              
            ;***RESET SCANLINE COUNT***
            ldy #192                ;[]+2   ;192 Scanlines
            ldx #0                  ;[]+2   ;Preload clear value for GRP0/1, enable FastFetch
            stx FASTFETCH           ;[]+4   ;Enable FastFetch mode
            ;*****************                
VB_LOOP:    lda INTIM               ;[]+4   ;Read timer
            bne VB_LOOP             ;[]+2/3 ;Loop if not 0
            sta VBLANK              ;[]+3   ;WSYNC happens after jump
            lda FrameNum            ;[]+3   ;Select which frame to draw
            bne *+5                 ;[]+3   ;Not Frame1?
            jmp FRAME1              ;[]+3   ;Go Frame1
            jmp FRAME2              ;[]+3   ;Go Frame2
; --------------------------------------------------------------------------------------------------
; [PLAYFIELD]* Starts drawing on 38th scanline
;            * 14,592 Cycles for use
;            * 76 per scanline
;            * 22.6 in Hblank
;            * 53.3 in Visible area
; --------------------------------------------------------------------------------------------------
;***********************************************************************
; [INTERLACED SCANLINE LOOP]
;            * 192 Scanlines total (96 draw scanlines + 96 logic lines)
;            * Scanline [38], S.cyc [0]
;            * PixelPos [-68], Color clock [0]
;***********************************************************************
          
;*************
;* [FRAME 1/2]
;*************
FRAME1:     sta WSYNC               ;[]+3     ;-PixelPos [-68], Color clock [0]
            dey                     ;[0]+2    ;Decrement scanline and compare to #0
            bne .doDraw1            ;[2]+2/3  ;If more scanlines left, branch
            jmp OVERSCAN            ;[4]+3    ;EXIT PLAYFIELD KERNAL
            ;***CYCLE EXACT DRAW KERNAL*** 
.doDraw1    sleep 19                ;[5]+19   ;Cycle padding
            lda #<DF1DATA           ;[24]+2 Load P0(0) 
            sta.w GRP0              ;[26]+4 Draw P0(0) 
            lda #<DF1DATA           ;[30]+2 Load P1(0) 
            sta.w GRP0              ;[32]+4 Draw P1(0)       
            lda #<DF1DATA           ;[36]+2 Load P0(1) 
            sta.w GRP0              ;[38]+4 Draw P0(1)             
            lda #<DF1DATA           ;[42]+2 Load P0(2) 
            sta.w GRP1              ;[44]+4 Draw P0(2) 
            lda #<DF1DATA           ;[48]+2 Load P1(1)       
            sta GRP1                ;[50]+4 Draw P1(1) 
            lda #<DF1DATA           ;[54]+2 Load P1(2) 
            sta GRP1                ;[56]+4 Draw P1(2)     
            ;***END DRAW***
            jmp FRAME1              ;[60]+3 ;Will WSYNC after jump
            
;*************
;* [FRAME 2/4]
;*************
FRAME2:     sta WSYNC               ;[]+3     ;-PixelPos [-68], Color clock [0]
            dey                     ;[0]+2    ;Decrement scanline and compare to #0
            bne .doDraw2            ;[2]+2/3  ;If more scanlines left, branch
            jmp OVERSCAN            ;[4]+3    ;EXIT PLAYFIELD KERNAL
            ;***CYCLE EXACT DRAW KERNAL*** 
.doDraw2    sleep 22                ;[5]+22   ;Cycle padding
            lda #<DF2DATA           ;[27]+2 Load P0(0) 
            sta.w GRP0              ;[29]+4 Draw P0(0) 
            lda #<DF2DATA           ;[33]+2 Load P0(1) 
            sta.w GRP0              ;[35]+4 Draw P0(1)       
            lda #<DF2DATA           ;[39]+2 Load P0(2) 
            sta.w GRP0              ;[41]+4 Draw P0(2)             
            lda #<DF2DATA           ;[45]+2 Load P1(0) 
            sta.w GRP1              ;[47]+4 Draw P1(0) 
            lda #<DF2DATA           ;[51]+2 Load P1(1)       
            sta GRP1                ;[53]+4 Draw P1(1) 
            lda #<DF2DATA           ;[57]+2 Load P1(2) 
            sta GRP1                ;[59]+4 Draw P1(2)    
            ;***END DRAW***            
            jmp FRAME2              ;[63]+3 ;Will WSYNC after jump      
       
; --------------------------------------------------------------------------------------------------
; [OVERSCAN];2280 Cycles for use
;           ;76 per scanline
;           ;30/29 scanlines
; --------------------------------------------------------------------------------------------------
OVERSCAN:   ;***DRAW KERNAL CLEANUP***
            stx GRP0                ;Clear GFX residue
            stx GRP1                ;-
            lda #$FF                ;Disable FastFetch mode        
            sta FASTFETCH           ;-
            ;***OVERSCAN TIME***   
            lda #$42                ;Disable beam/Start VBLANK
            sta VBLANK              ;-
            lda #35                 ;Set timer for 30 scanlines
            sta TIM64T              ;-
            ;***SETUP KERNAL FRAMES***
            lda FrameNum            ;Frame switch
            eor #$1                 ;Prepare for next frame
            sta FrameNum            ;Store new frame
            bne *+8                 ;Set screen for Frame2?                        
            jsr Position1           ;Position Frame1
            jmp *+6                 ;Forced Branch  
            jsr Position2           ;Position Frame2
            ;*****************
            ;**** [LOGIC] ****
            ;*****************    
            jsr CheckSwitches       ;Get input data                   
            jsr CheckBounds         ;Software collision detection
            jsr GeneratePos         ;Get new positional data
            jsr GenerateP0          ;Update Player GFX
            ;*****************
.OS_loop:   lda INTIM               ;Read timer
            bne .OS_loop            ;-loop until 0
            sta WSYNC               ;Rollover to scanline 258
            jmp KERNAL              ;-Jump up to KERNAL/VSYNC to process last 3 scanlines

; --------------------------------------------------------------------------------------------------
; [LOGIC ROUTINES SECTION]
; --------------------------------------------------------------------------------------------------
;********************************
;     [HORIZONTAL POSITIONING]
;********************************
Position1: 
	      sta WSYNC
        sleep 29
      	sta RESP0            
      	lda #$20
      	sta HMP0
      	sta HMP1
      	sta WSYNC
	      sleep 45
      	sta RESP1            
      	lda #$10
      	sta WSYNC
      	sta HMOVE
      	sta WSYNC
      	sta HMCLR
	      rts
	      
Position2:          
	      sta WSYNC
	      sleep 32
      	sta RESP0            
      	lda #$30
      	sta HMP0
      	sta HMP1
      	sta WSYNC
	      sleep 48
      	sta RESP1            
      	lda #$10
      	sta WSYNC
      	sta HMOVE
      	sta WSYNC
      	sta HMCLR
	      rts
	      
;********************************
;     [CHECK INPUT]
;********************************
CheckSwitches:
      ;Check RESET switch
      lda SWCHB       ;[]+3 
      lsr             ;[]+2  Check RESET switch
      bcs .noRST      ;[]2/3 0 = pressed
      jmp RESET       ;[]+3  pressed, reset console
.noRST
      lda SWCHA       ;[]+4
      tay             ;[]+2
      bmi .M0         ;[]+2
      inc Player0X    ;[]+5  ;move right
.M0:  tya             ;[]+2
      and #$40        ;[]+2
      bne .M1         ;[]+2
      dec Player0X    ;[]+5  ;move left
.M1:  tya             ;[]+2
      and #$20        ;[]+2
      bne .M2         ;[]+2
      inc Player0Y    ;[]+5  ;move down
.M2:  tya             ;[]+2
      and #$10        ;[]+2
      bne .M3         ;[]+2
      dec Player0Y    ;[]+5  ;move up
.M3:  lda INPT4       ;[]+4  ;Fire Button
      bmi .M4         ;[]+3
      nop
.M4:  rts 

;********************************
;     [COLLISION DETECTION]
;********************************
;Will need to be determined later
CheckBounds:
    lda Player0X
    cmp #255
    bne .skipL
    lda #0
    sta Player0X
.skipL
    lda Player0X
    cmp #89
    bne .skipR
    lda #88
    sta Player0X
.skipR
    lda Player0Y
    cmp #255
    bne .skipT
    lda #0
    sta Player0Y
.skipT
    lda Player0Y
    cmp #168
    bne .skipB
    lda #167
    sta Player0Y
.skipB
    rts
    
;********************************
;     [GENERATE DATA]
;********************************          
GeneratePos:
    ;* Set P0pointer to ScreenRAM space
    ;* if drawing Frame2 data set pointer to ScreenRAM2 offset
    ;* Add players X position to low byte of pointer
    lda #$00                ;Clear pointer
    sta P0pointer           ;-
    sta P0pointer+1         ;-
    lda Player0Y            ;Get Player Scanline position
    beq .SLzero             ;Scanline #0, skip multiply
    ldy #$04                ;Multiply by 16    
    clc                     ;Clear carry
.mul6
    adc Player0Y            ;Add Scanline number position
    bcc .noCarry            ;No carry, add again
    inc P0pointer+1         ;Add carry to HI byte   
    clc                     ;Clear carry
.noCarry
    dey                     ;Loop
    bpl .mul6               ;-
    sta P0pointer           ;Set LOW byte, result of addition
.SLzero
    ;Adjust pointer to player X position
    lda Player0X            ;Load X offset
    lsr                     ;-2 Divide by 16 to get byte offset
    lsr                     ;-4
    lsr                     ;-8
    lsr                     ;-16
    tax                     ;Hold result
    bcc .SR1                ;If even skip +1152 adjustment
    clc                     ;Clear carry
    ;ScreenRAM2
    adc #<ScreenRAM2        ;Add offset +1152 to ScreenRAM2 LOW address
    adc P0pointer           ;Add current LOW byte offset
    sta P0pointer           ;Store LOW
    lda #>ScreenRAM2        ;Load HI byte offset amount 
    adc P0pointer+1         ;Add to HI byte (+1 if carry set from LOW)
    sta P0pointer+1         ;Store HI
    rts                     ;Done
.SR1
    clc                     ;Clear Carry
    txa                     ;Restore slot number  
    adc P0pointer           ;Add current LOW byte offset
    sta P0pointer           ;Store LOW
    lda #>ScreenRAM1        ;Load HI byte offset amount
    adc P0pointer+1         ;Add to HI byte (+1 if carry set from LOW)
    sta P0pointer+1         ;Store HI
    rts                     ;Done
    
GenerateP0:    
    lda #<P0Sprite          ;Load LOW sprite address
    sta SPRpoint            ;Store LOW
    lda #>P0Sprite          ;Load HI sprite address
    sta SPRpoint+1          ;Store HI
    ;Update Player Graphics into ScreenRAM
    lda P0pointer           ;Load Player LOW address in ScreenRAM
    sta DF0LOW              ;Set DF0LOW
    lda P0pointer+1         ;Load Player HI address in ScreenRAM
    sta DF0HI               ;Set DF0HI
    ldy #7                  ;8 bytes of sprite 7 to 0
.loadP0
    lda (SPRpoint),Y        ;Get sprite via pointer
    sta DF0WRITE            ;Write to ScreenRAM
    ;Add 6 to DF0LOW
    lda #6                  ;Offset by 6 bytes for next scanline
    clc                     ;Ensure carry is clear
    adc P0pointer           ;Add 6 bytes to get next scanline address
    sta P0pointer           ;Set new LOW pointer
    sta DF0LOW              ;Update LOW pointer    
    bcc .resume             ;Branch on carry clear
    inc P0pointer+1         ;Add carry to HI pointer
    lda P0pointer+1         ;Get new HI address
    sta DF0HI               ;Update HI pointer
.resume    
    dey                     ;Next sprite byte
    bpl .loadP0             ;Finished?
    rts                     ;DONE
  
;********************************
;     [SPRITE GRAPHICS]
;******************************** 
P0Sprite:
  .byte #$FF
  .byte #$FF 
  .byte #$FF
  .byte #$FF 
  .byte #$FF
  .byte #$FF 
  .byte #$FF
  .byte #$FF 
  
;********************
  ORG  $0FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 0]---
;///////////////////// 

;--------------------------------------------------------------------------------------------------
;*********************
; [START OF BANK 1]
;*********************
  ORG  $1000                        ;Relocatable ORG address
  RORG $1000                        ;-
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank1Code:
  nop
;********************
  ORG  $1FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 1]---
;/////////////////////  

;--------------------------------------------------------------------------------------------------
;*********************
; [START OF BANK 2]
;*********************
  ORG  $2000                        ;Relocatable ORG address
  RORG $1000                        ;-
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank2Code:
  nop	
;********************
  ORG  $2FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 2]---
;///////////////////// 

;--------------------------------------------------------------------------------------------------
;*********************
; [START OF BANK 3]
;*********************
  ORG  $3000                        ;Relocatable ORG address
  RORG $1000                        ;-
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank3Code:
  nop		
;********************
  ORG  $3FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 3]---
;///////////////////// 

;--------------------------------------------------------------------------------------------------
;*********************
; [START OF BANK 4]
;*********************
  ORG  $4000                        ;Relocatable ORG address
  RORG $1000                        ;-
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank4Code:
  nop	
;********************
  ORG  $4FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 4]---
;///////////////////// 

;--------------------------------------------------------------------------------------------------
;*********************
; [START OF BANK 5]
;*********************
  ORG  $5000                        ;Relocatable ORG address
  RORG $1000                        ;-
  JUMP_TABLE                        ;Required Jump table for bank switching
;********************
Bank5Code:
  nop
;********************
  ORG  $5FF6                        ;Relocatable ORG address
  RORG $1FF6                        ;-
  BANKS_AND_VECTORS                 ;Required Jump table for bank switching
;/////////////////////
;---[END OF BANK 5]---
;/////////////////////

;********************************
;   [*GRAPHICS ONLY* BANK 6]
;********************************
  ORG $6000
  RORG $0000   
  
ScreenRAM1: ds 1152
ScreenRAM2: ds 1152

ROMpadding ds ($1400 - *)
  ;BANK 6 100% full